'use strict';

var Promise = require("bluebird");

/* exported logger */

var logger = require('./logger');

var OriginalServerConnector = require('../../main/js/ServerConnector');
var NetworkError = require('../../main/js/NetworkError');

var fs = require('fs');
var request = require('request');

var ServerConnectorMock = OriginalServerConnector;

function replaceAsterisk(str) {
  return str.replace(/\*/g,"all").replace(/\:/g,".");
}
function urlToFileName(url) {
  var result = url;
  var token = OriginalServerConnector.getSessionData().getToken();
  if (token!==undefined && token!=="" && url.startsWith("./testFiles/apiCalls")) {
    if (!result.endsWith("&") &&!result.endsWith("_") ) {
      result +="/";
    }
    result+="token=" +token+"&";
  }
  		
  return replaceAsterisk(result);
}

ServerConnectorMock._readFile = function(url) {
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.get(url, function(error, response, body) {
        if (error) {
          reject(error);
        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var fileName = urlToFileName(url);
      fs.readFile(fileName, 'utf8', function(err, content) {
        if (err) {
          reject(new NetworkError(err.message, {
            content : content,
            url : fileName,
            statusCode : 404
          }));
        } else {
          resolve(content);
        }
      });
    }
  });
};

function encodeParams(params) {
  var result = {};
  for ( var key in params) {
    if (params.hasOwnProperty(key)) {
      result[key] = encodeURIComponent(params[key]);
    }
  }
  return result;
}

ServerConnectorMock.sendPostRequest = function(url, params) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.post({
        url : url,
        form : params
      }, function(error, response, body) {
        if (error) {
          reject(error);

        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var mockUrl = urlToFileName(url + "/POST_" + self.createGetParams(encodeParams(params)));
      fs.readFile(mockUrl, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(content);
        }
      });
    }
  });
};

ServerConnectorMock.sendPutRequest = function(url, params) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.put({
        url : url,
        form : params
      }, function(error, response, body) {
        if (error) {
          reject(error);

        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var mockUrl = urlToFileName(url + "/PUT_" + self.createGetParams(encodeParams(params)));
      fs.readFile(mockUrl, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(content);
        }
      });
    }
  });
};

ServerConnectorMock.sendDeleteRequest = function(url, params) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.delete({
        url : url,
        form : params
      }, function(error, response, body) {
        if (error) {
          reject(error);

        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var mockUrl = urlToFileName(url + "/DELETE_" + self.createGetParams(encodeParams(params)));
      fs.readFile(mockUrl, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(JSON.parse(content));
        }
      });
    }
  });
};

ServerConnectorMock.sendPatchRequest = function(url, params) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.delete({
        url : url,
        form : params
      }, function(error, response, body) {
        if (error) {
          reject(error);

        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var mockUrl = urlToFileName(url + "/PATCH_" + self.createGetParams(params));
      fs.readFile(mockUrl, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(JSON.parse(content));
        }
      });
    }
  });
};

ServerConnectorMock.getApiBaseUrl = function() {
  return "./testFiles/apiCalls/";
};

var originalGetApiUrl = OriginalServerConnector.getApiUrl;

ServerConnectorMock.getApiUrl = function(paramObj) {
  // replace '?' with '/'
  // the call is done on ServerConnectorObject (so 'this' is set properly)
  return originalGetApiUrl.call(this, paramObj).replace(/\?/g, '/');
};

module.exports = ServerConnectorMock;
